/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 1.3.24
 * 
 * This file is not intended to be easily readable and contains a number of 
 * coding conventions designed to improve portability and efficiency. Do not make
 * changes to this file unless you know what you are doing--modify the SWIG 
 * interface file instead. 
 * ----------------------------------------------------------------------------- */


#ifndef SWIG_TEMPLATE_DISAMBIGUATOR
#  if defined(__SUNPRO_CC) 
#    define SWIG_TEMPLATE_DISAMBIGUATOR template
#  else
#    define SWIG_TEMPLATE_DISAMBIGUATOR 
#  endif
#endif

/***********************************************************************
 * swigrun.swg
 *
 *     This file contains generic CAPI SWIG runtime support for pointer
 *     type checking.
 *
 ************************************************************************/

/* This should only be incremented when either the layout of swig_type_info changes,
   or for whatever reason, the runtime changes incompatibly */
#define SWIG_RUNTIME_VERSION "1"

/* define SWIG_TYPE_TABLE_NAME as "SWIG_TYPE_TABLE" */
#ifdef SWIG_TYPE_TABLE
#define SWIG_QUOTE_STRING(x) #x
#define SWIG_EXPAND_AND_QUOTE_STRING(x) SWIG_QUOTE_STRING(x)
#define SWIG_TYPE_TABLE_NAME SWIG_EXPAND_AND_QUOTE_STRING(SWIG_TYPE_TABLE)
#else
#define SWIG_TYPE_TABLE_NAME
#endif

#include <string.h>

#ifndef SWIGINLINE
#if defined(__cplusplus) || (defined(__GNUC__) && !defined(__STRICT_ANSI__))
#  define SWIGINLINE inline
#else
#  define SWIGINLINE
#endif
#endif

/*
  You can use the SWIGRUNTIME and SWIGRUNTIMEINLINE macros for
  creating a static or dynamic library from the swig runtime code.
  In 99.9% of the cases, swig just needs to declare them as 'static'.
  
  But only do this if is strictly necessary, ie, if you have problems
  with your compiler or so.
*/
#ifndef SWIGRUNTIME
#define SWIGRUNTIME static
#endif
#ifndef SWIGRUNTIMEINLINE
#define SWIGRUNTIMEINLINE SWIGRUNTIME SWIGINLINE
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef void *(*swig_converter_func)(void *);
typedef struct swig_type_info *(*swig_dycast_func)(void **);

typedef struct swig_type_info {
  const char             *name;
  swig_converter_func     converter;
  const char             *str;
  void                   *clientdata;
  swig_dycast_func        dcast;
  struct swig_type_info  *next;
  struct swig_type_info  *prev;
} swig_type_info;

/* 
  Compare two type names skipping the space characters, therefore
  "char*" == "char *" and "Class<int>" == "Class<int >", etc.

  Return 0 when the two name types are equivalent, as in
  strncmp, but skipping ' '.
*/
SWIGRUNTIME int
SWIG_TypeNameComp(const char *f1, const char *l1,
		  const char *f2, const char *l2) {
  for (;(f1 != l1) && (f2 != l2); ++f1, ++f2) {
    while ((*f1 == ' ') && (f1 != l1)) ++f1;
    while ((*f2 == ' ') && (f2 != l2)) ++f2;
    if (*f1 != *f2) return *f1 - *f2;
  }
  return (l1 - f1) - (l2 - f2);
}

/*
  Check type equivalence in a name list like <name1>|<name2>|...
*/
SWIGRUNTIME int
SWIG_TypeEquiv(const char *nb, const char *tb) {
  int equiv = 0;
  const char* te = tb + strlen(tb);
  const char* ne = nb;
  while (!equiv && *ne) {
    for (nb = ne; *ne; ++ne) {
      if (*ne == '|') break;
    }
    equiv = SWIG_TypeNameComp(nb, ne, tb, te) == 0;
    if (*ne) ++ne;
  }
  return equiv;
}

/*
  Register a type mapping with the type-checking
*/
SWIGRUNTIME swig_type_info *
SWIG_TypeRegisterTL(swig_type_info **tl, swig_type_info *ti) {
  swig_type_info *tc, *head, *ret, *next;
  /* Check to see if this type has already been registered */
  tc = *tl;
  while (tc) {
    /* check simple type equivalence */
    int typeequiv = (strcmp(tc->name, ti->name) == 0);   
    /* check full type equivalence, resolving typedefs */
    if (!typeequiv) {
      /* only if tc is not a typedef (no '|' on it) */
      if (tc->str && ti->str && !strstr(tc->str,"|")) {
	typeequiv = SWIG_TypeEquiv(ti->str,tc->str);
      }
    }
    if (typeequiv) {
      /* Already exists in the table.  Just add additional types to the list */
      if (ti->clientdata) tc->clientdata = ti->clientdata;
      head = tc;
      next = tc->next;
      goto l1;
    }
    tc = tc->prev;
  }
  head = ti;
  next = 0;

  /* Place in list */
  ti->prev = *tl;
  *tl = ti;

  /* Build linked lists */
  l1:
  ret = head;
  tc = ti + 1;
  /* Patch up the rest of the links */
  while (tc->name) {
    head->next = tc;
    tc->prev = head;
    head = tc;
    tc++;
  }
  if (next) next->prev = head;
  head->next = next;

  return ret;
}

/*
  Check the typename
*/
SWIGRUNTIME swig_type_info *
SWIG_TypeCheck(const char *c, swig_type_info *ty) {
  swig_type_info *s;
  if (!ty) return 0;        /* Void pointer */
  s = ty->next;             /* First element always just a name */
  do {
    if (strcmp(s->name,c) == 0) {
      if (s == ty->next) return s;
      /* Move s to the top of the linked list */
      s->prev->next = s->next;
      if (s->next) {
        s->next->prev = s->prev;
      }
      /* Insert s as second element in the list */
      s->next = ty->next;
      if (ty->next) ty->next->prev = s;
      ty->next = s;
      s->prev = ty;
      return s;
    }
    s = s->next;
  } while (s && (s != ty->next));
  return 0;
}

/*
  Cast a pointer up an inheritance hierarchy
*/
SWIGRUNTIMEINLINE void *
SWIG_TypeCast(swig_type_info *ty, void *ptr) {
  return ((!ty) || (!ty->converter)) ? ptr : (*ty->converter)(ptr);
}

/* 
   Dynamic pointer casting. Down an inheritance hierarchy
*/
SWIGRUNTIME swig_type_info *
SWIG_TypeDynamicCast(swig_type_info *ty, void **ptr) {
  swig_type_info *lastty = ty;
  if (!ty || !ty->dcast) return ty;
  while (ty && (ty->dcast)) {
    ty = (*ty->dcast)(ptr);
    if (ty) lastty = ty;
  }
  return lastty;
}

/*
  Return the name associated with this type
*/
SWIGRUNTIMEINLINE const char *
SWIG_TypeName(const swig_type_info *ty) {
  return ty->name;
}

/*
  Return the pretty name associated with this type,
  that is an unmangled type name in a form presentable to the user.
*/
SWIGRUNTIME const char *
SWIG_TypePrettyName(const swig_type_info *type) {
  /* The "str" field contains the equivalent pretty names of the
     type, separated by vertical-bar characters.  We choose
     to print the last name, as it is often (?) the most
     specific. */
  if (type->str != NULL) {
    const char *last_name = type->str;
    const char *s;
    for (s = type->str; *s; s++)
      if (*s == '|') last_name = s+1;
    return last_name;
  }
  else
    return type->name;
}

/*
  Search for a swig_type_info structure
*/
SWIGRUNTIME swig_type_info *
SWIG_TypeQueryTL(swig_type_info *tl, const char *name) {
  swig_type_info *ty = tl;
  while (ty) {
    if (ty->str && (SWIG_TypeEquiv(ty->str,name))) return ty;
    if (ty->name && (strcmp(name,ty->name) == 0)) return ty;
    ty = ty->prev;
  }
  return 0;
}

/* 
   Set the clientdata field for a type
*/
SWIGRUNTIME void
SWIG_TypeClientDataTL(swig_type_info *tl, swig_type_info *ti, void *clientdata) {
  swig_type_info *tc, *equiv;
  if (ti->clientdata) return;
  /* if (ti->clientdata == clientdata) return; */
  ti->clientdata = clientdata;
  equiv = ti->next;
  while (equiv) {
    if (!equiv->converter) {
      tc = tl;
      while (tc) {
        if ((strcmp(tc->name, equiv->name) == 0))
          SWIG_TypeClientDataTL(tl,tc,clientdata);
        tc = tc->prev;
      }
    }
    equiv = equiv->next;
  }
}

/* 
   Pack binary data into a string
*/
SWIGRUNTIME char *
SWIG_PackData(char *c, void *ptr, size_t sz) {
  static char hex[17] = "0123456789abcdef";
  unsigned char *u = (unsigned char *) ptr;
  const unsigned char *eu =  u + sz;
  register unsigned char uu;
  for (; u != eu; ++u) {
    uu = *u;
    *(c++) = hex[(uu & 0xf0) >> 4];
    *(c++) = hex[uu & 0xf];
  }
  return c;
}

/* 
   Unpack binary data from a string
*/
SWIGRUNTIME const char *
SWIG_UnpackData(const char *c, void *ptr, size_t sz) {
  register unsigned char *u = (unsigned char *) ptr;
  register const unsigned char *eu =  u + sz;
  for (; u != eu; ++u) {
    register int d = *(c++);
    register unsigned char uu = 0;
    if ((d >= '0') && (d <= '9'))
      uu = ((d - '0') << 4);
    else if ((d >= 'a') && (d <= 'f'))
      uu = ((d - ('a'-10)) << 4);
    else 
      return (char *) 0;
    d = *(c++);
    if ((d >= '0') && (d <= '9'))
      uu |= (d - '0');
    else if ((d >= 'a') && (d <= 'f'))
      uu |= (d - ('a'-10));
    else 
      return (char *) 0;
    *u = uu;
  }
  return c;
}

/*
  This function will propagate the clientdata field of type to any new
  swig_type_info structures that have been added into the list of
  equivalent types.  It is like calling SWIG_TypeClientData(type,
  clientdata) a second time.
*/
SWIGRUNTIME void
SWIG_PropagateClientDataTL(swig_type_info *tl, swig_type_info *type) {
  swig_type_info *equiv = type->next;
  swig_type_info *tc;
  if (!type->clientdata) return;
  while (equiv) {
    if (!equiv->converter) {
      tc = tl;
      while (tc) {
        if ((strcmp(tc->name, equiv->name) == 0) && !tc->clientdata)
          SWIG_TypeClientDataTL(tl,tc, type->clientdata);
        tc = tc->prev;
      }
    }
    equiv = equiv->next;
  }
}

/* 
   Pack 'void *' into a string buffer.
*/
SWIGRUNTIME char *
SWIG_PackVoidPtr(char *buff, void *ptr, const char *name, size_t bsz) {
  char *r = buff;
  if ((2*sizeof(void *) + 2) > bsz) return 0;
  *(r++) = '_';
  r = SWIG_PackData(r,&ptr,sizeof(void *));
  if (strlen(name) + 1 > (bsz - (r - buff))) return 0;
  strcpy(r,name);
  return buff;
}

SWIGRUNTIME const char *
SWIG_UnpackVoidPtr(const char *c, void **ptr, const char *name) {
  if (*c != '_') {
    if (strcmp(c,"NULL") == 0) {
      *ptr = (void *) 0;
      return name;
    } else {
      return 0;
    }
  }
  return SWIG_UnpackData(++c,ptr,sizeof(void *));
}

SWIGRUNTIME char *
SWIG_PackDataName(char *buff, void *ptr, size_t sz, const char *name, size_t bsz) {
  char *r = buff;
  size_t lname = (name ? strlen(name) : 0);
  if ((2*sz + 2 + lname) > bsz) return 0;
  *(r++) = '_';
  r = SWIG_PackData(r,ptr,sz);
  if (lname) {
    strncpy(r,name,lname+1);
  } else {
    *r = 0;
  }
  return buff;
}

SWIGRUNTIME const char *
SWIG_UnpackDataName(const char *c, void *ptr, size_t sz, const char *name) {
  if (*c != '_') {
    if (strcmp(c,"NULL") == 0) {
      memset(ptr,0,sz);
      return name;
    } else {
      return 0;
    }
  }
  return SWIG_UnpackData(++c,ptr,sz);
}

#ifdef __cplusplus
}
#endif

/***********************************************************************
 * common.swg
 *
 *     This file contains generic SWIG runtime support for pointer
 *     type checking as well as a few commonly used macros to control
 *     external linkage.
 *
 * Author : David Beazley (beazley@cs.uchicago.edu)
 *
 * Copyright (c) 1999-2000, The University of Chicago
 * 
 * This file may be freely redistributed without license or fee provided
 * this copyright message remains intact.
 ************************************************************************/


#if defined(_WIN32) || defined(__WIN32__) || defined(__CYGWIN__)
#  if !defined(STATIC_LINKED)
#    define SWIGEXPORT(a) __declspec(dllexport) a
#  else
#    define SWIGEXPORT(a) a
#  endif
#else
#  define SWIGEXPORT(a) a
#endif

#ifdef __cplusplus
extern "C" {
#endif


/*************************************************************************/


/* The static type info list */

static swig_type_info *swig_type_list = 0;
static swig_type_info **swig_type_list_handle = &swig_type_list;
  

/* Register a type mapping with the type-checking */
static swig_type_info *
SWIG_TypeRegister(swig_type_info *ti) {
  return SWIG_TypeRegisterTL(swig_type_list_handle, ti);
}

/* Search for a swig_type_info structure */
static swig_type_info *
SWIG_TypeQuery(const char *name) {
  return SWIG_TypeQueryTL(*swig_type_list_handle, name);
}

/* Set the clientdata field for a type */
static void
SWIG_TypeClientData(swig_type_info *ti, void *clientdata) {
  SWIG_TypeClientDataTL(*swig_type_list_handle, ti, clientdata);
}

/* This function will propagate the clientdata field of type to
* any new swig_type_info structures that have been added into the list
* of equivalent types.  It is like calling
* SWIG_TypeClientData(type, clientdata) a second time.
*/
static void
SWIG_PropagateClientData(swig_type_info *type) {
  SWIG_PropagateClientDataTL(*swig_type_list_handle, type);
}

#ifdef __cplusplus
}
#endif

/* ---------------------------------------------------------------------- -*- c -*-
 * perl5.swg
 *
 * Perl5 runtime library
 * $Header: /cvsroot/swig/SWIG/Lib/perl5/perlrun.swg,v 1.20 2004/11/29 23:13:57 wuzzeb Exp $
 * ----------------------------------------------------------------------------- */

#define SWIGPERL
#define SWIGPERL5
#ifdef __cplusplus
/* Needed on some windows machines---since MS plays funny games with the header files under C++ */
#include <math.h>
#include <stdlib.h>
extern "C" {
#endif
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

/* Get rid of free and malloc defined by perl */
#undef free
#undef malloc

#ifndef pTHX_
#define pTHX_
#endif

#include <string.h>
#ifdef __cplusplus
}
#endif

/* Macro to call an XS function */

#ifdef PERL_OBJECT 
#  define SWIG_CALLXS(_name) _name(cv,pPerl) 
#else 
#  ifndef MULTIPLICITY 
#    define SWIG_CALLXS(_name) _name(cv) 
#  else 
#    define SWIG_CALLXS(_name) _name(PERL_GET_THX, cv) 
#  endif 
#endif 

/* Contract support */

#define SWIG_contract_assert(expr,msg) if (!(expr)) { SWIG_croak(msg); } else

/* Note: SwigMagicFuncHack is a typedef used to get the C++ compiler to just shut up already */

#ifdef PERL_OBJECT
#define MAGIC_PPERL  CPerlObj *pPerl = (CPerlObj *) this;
typedef int (CPerlObj::*SwigMagicFunc)(SV *, MAGIC *);

#ifdef __cplusplus
extern "C" {
#endif
typedef int (CPerlObj::*SwigMagicFuncHack)(SV *, MAGIC *);
#ifdef __cplusplus
}
#endif

#define SWIG_MAGIC(a,b) (SV *a, MAGIC *b)
#define SWIGCLASS_STATIC
#else
#define MAGIC_PPERL
#define SWIGCLASS_STATIC static
#ifndef MULTIPLICITY
#define SWIG_MAGIC(a,b) (SV *a, MAGIC *b)
typedef int (*SwigMagicFunc)(SV *, MAGIC *);

#ifdef __cplusplus
extern "C" {
#endif
typedef int (*SwigMagicFuncHack)(SV *, MAGIC *);
#ifdef __cplusplus
}
#endif


#else
#define SWIG_MAGIC(a,b) (struct interpreter *interp, SV *a, MAGIC *b)
typedef int (*SwigMagicFunc)(struct interpreter *, SV *, MAGIC *);
#ifdef __cplusplus
extern "C" {
#endif
typedef int (*SwigMagicFuncHack)(struct interpreter *, SV *, MAGIC *);
#ifdef __cplusplus
}
#endif

#endif
#endif

#if defined(WIN32) && defined(PERL_OBJECT) && !defined(PerlIO_exportFILE)
#define PerlIO_exportFILE(fh,fl) (FILE*)(fh)
#endif

/* Modifications for newer Perl 5.005 releases */

#if !defined(PERL_REVISION) || ((PERL_REVISION >= 5) && ((PERL_VERSION < 5) || ((PERL_VERSION == 5) && (PERL_SUBVERSION < 50))))
#  ifndef PL_sv_yes
#    define PL_sv_yes sv_yes
#  endif
#  ifndef PL_sv_undef
#    define PL_sv_undef sv_undef
#  endif
#  ifndef PL_na
#    define PL_na na
#  endif
#endif

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

#define SWIG_OWNER 1
#define SWIG_SHADOW 2

/* Common SWIG API */

#ifdef PERL_OBJECT
#  define SWIG_ConvertPtr(obj, pp, type, flags) \
     SWIG_Perl_ConvertPtr(pPerl, obj, pp, type, flags)
#  define SWIG_NewPointerObj(p, type, flags) \
     SWIG_Perl_NewPointerObj(pPerl, p, type, flags)
#  define SWIG_MakePackedObj(sv, p, s, type)	\
     SWIG_Perl_MakePackedObj(pPerl, sv, p, s, type)
#  define SWIG_ConvertPacked(obj, p, s, type, flags) \
     SWIG_Perl_ConvertPacked(pPerl, obj, p, s, type, flags)

#else
#  define SWIG_ConvertPtr(obj, pp, type, flags) \
     SWIG_Perl_ConvertPtr(obj, pp, type, flags)
#  define SWIG_NewPointerObj(p, type, flags) \
     SWIG_Perl_NewPointerObj(p, type, flags)
#  define SWIG_MakePackedObj(sv, p, s, type)	\
     SWIG_Perl_MakePackedObj(sv, p, s, type )
#  define SWIG_ConvertPacked(obj, p, s, type, flags) \
     SWIG_Perl_ConvertPacked(obj, p, s, type, flags)
#endif

/* Perl-specific API */
#ifdef PERL_OBJECT
#  define SWIG_MakePtr(sv, ptr, type, flags) \
     SWIG_Perl_MakePtr(pPerl, sv, ptr, type, flags)
#  define SWIG_SetError(str) \
     SWIG_Perl_SetError(pPerl, str)
#else
#  define SWIG_MakePtr(sv, ptr, type, flags) \
     SWIG_Perl_MakePtr(sv, ptr, type, flags)
#  define SWIG_SetError(str) \
     SWIG_Perl_SetError(str)
#  define SWIG_SetErrorSV(str) \
     SWIG_Perl_SetErrorSV(str)
#endif

#define SWIG_SetErrorf SWIG_Perl_SetErrorf


#ifdef PERL_OBJECT
#  define SWIG_MAYBE_PERL_OBJECT CPerlObj *pPerl,
#else
#  define SWIG_MAYBE_PERL_OBJECT
#endif

static swig_type_info **
SWIG_Perl_GetTypeListHandle() {
  static void *type_pointer = (void *)0;
  SV *pointer;

  /* first check if pointer already created */
  if (!type_pointer) {
    pointer = get_sv("swig_runtime_data::type_pointer" SWIG_RUNTIME_VERSION SWIG_TYPE_TABLE_NAME, FALSE);
    if (pointer && SvOK(pointer)) {
      type_pointer = INT2PTR(swig_type_info **, SvIV(pointer));
    }
  }

  return (swig_type_info **) type_pointer;
}

/*
  Search for a swig_type_info structure
 */
SWIGRUNTIMEINLINE swig_type_info *
SWIG_Perl_GetTypeList() {
  swig_type_info **tlh = SWIG_Perl_GetTypeListHandle();
  return tlh ? *tlh : (swig_type_info*)0;
}

#define SWIG_Runtime_GetTypeList SWIG_Perl_GetTypeList 

static swig_type_info *
SWIG_Perl_TypeCheckRV(SWIG_MAYBE_PERL_OBJECT SV *rv, swig_type_info *ty) {
  swig_type_info *s;
  if (!ty) return 0;        /* Void pointer */
  s = ty->next;             /* First element always just a name */
  do {
    if (sv_derived_from(rv, (char *) s->name)) {
      if (s == ty->next) return s;
      /* Move s to the top of the linked list */
      s->prev->next = s->next;
      if (s->next) {
        s->next->prev = s->prev;
      }
      /* Insert s as second element in the list */
      s->next = ty->next;
      if (ty->next) ty->next->prev = s;
      ty->next = s;
      s->prev = ty;
      return s;
    }
    s = s->next;
  } while (s && (s != ty->next));
  return 0;
}

/* Function for getting a pointer value */

static int
SWIG_Perl_ConvertPtr(SWIG_MAYBE_PERL_OBJECT SV *sv, void **ptr, swig_type_info *_t, int flags) {
  swig_type_info *tc;
  void *voidptr = (void *)0;

  /* If magical, apply more magic */
  if (SvGMAGICAL(sv))
    mg_get(sv);

  /* Check to see if this is an object */
  if (sv_isobject(sv)) {
    SV *tsv = (SV*) SvRV(sv);
    IV tmp = 0;
    if ((SvTYPE(tsv) == SVt_PVHV)) {
      MAGIC *mg;
      if (SvMAGICAL(tsv)) {
        mg = mg_find(tsv,'P');
        if (mg) {
          sv = mg->mg_obj;
          if (sv_isobject(sv)) {
            tmp = SvIV((SV*)SvRV(sv));
          }
        }
      } else {
        return -1;
      }
    } else {
      tmp = SvIV((SV*)SvRV(sv));
    }
    voidptr = (void *)tmp;
    if (!_t) {
      *(ptr) = voidptr;
      return 0;
    }
  } else if (! SvOK(sv)) {            /* Check for undef */
    *(ptr) = (void *) 0;
    return 0;
  } else if (SvTYPE(sv) == SVt_RV) {  /* Check for NULL pointer */
    *(ptr) = (void *) 0;
    if (!SvROK(sv))
      return 0;
    else
      return -1;
  } else {                            /* Don't know what it is */
    *(ptr) = (void *) 0;
    return -1;
  }
  if (_t) {
    /* Now see if the types match */
    char *_c = HvNAME(SvSTASH(SvRV(sv)));
    tc = SWIG_TypeCheck(_c,_t);
    if (!tc) {
      *ptr = voidptr;
      return -1;
    }
    *ptr = SWIG_TypeCast(tc,voidptr);
    return 0;
  }
  *ptr = voidptr;
  return 0;
}

static void
SWIG_Perl_MakePtr(SWIG_MAYBE_PERL_OBJECT SV *sv, void *ptr, swig_type_info *t, int flags) {
  if (ptr && (flags & SWIG_SHADOW)) {
    SV *self;
    SV *obj=newSV(0);
    HV *hash=newHV();
    HV *stash;
    sv_setref_pv(obj, (char *) t->name, ptr);
    stash=SvSTASH(SvRV(obj));
    if (flags & SWIG_OWNER) {
      HV *hv;
      GV *gv=*(GV**)hv_fetch(stash, "OWNER", 5, TRUE);
      if (!isGV(gv))
        gv_init(gv, stash, "OWNER", 5, FALSE);
      hv=GvHVn(gv);
      hv_store_ent(hv, obj, newSViv(1), 0);
    }
    sv_magic((SV *)hash, (SV *)obj, 'P', Nullch, 0);
    SvREFCNT_dec(obj);
    self=newRV_noinc((SV *)hash);
    sv_setsv(sv, self);
    SvREFCNT_dec((SV *)self);
    sv_bless(sv, stash);
  }
  else {
    sv_setref_pv(sv, (char *) t->name, ptr);
  }
}

static SWIGINLINE SV *
SWIG_Perl_NewPointerObj(SWIG_MAYBE_PERL_OBJECT void *ptr, swig_type_info *t, int flags) {
  SV *result = sv_newmortal();
  SWIG_MakePtr(result, ptr, t, flags);
  return result;
}

static void
  SWIG_Perl_MakePackedObj(SWIG_MAYBE_PERL_OBJECT SV *sv, void *ptr, int sz, swig_type_info *type) {
  char result[1024];
  char *r = result;
  if ((2*sz + 1 + strlen(type->name)) > 1000) return;
  *(r++) = '_';
  r = SWIG_PackData(r,ptr,sz);
  strcpy(r,type->name);
  sv_setpv(sv, result);
}

/* Convert a packed value value */
static int
SWIG_Perl_ConvertPacked(SWIG_MAYBE_PERL_OBJECT SV *obj, void *ptr, int sz, swig_type_info *ty, int flags) {
  swig_type_info *tc;
  const char  *c = 0;

  if ((!obj) || (!SvOK(obj))) return -1;
  c = SvPV(obj, PL_na);
  /* Pointer values must start with leading underscore */
  if (*c != '_') return -1;
  c++;
  c = SWIG_UnpackData(c,ptr,sz);
  if (ty) {
    tc = SWIG_TypeCheck(c,ty);
    if (!tc) return -1;
  }
  return 0;
}

static SWIGINLINE void
SWIG_Perl_SetError(SWIG_MAYBE_PERL_OBJECT const char *error) {
  if (error) sv_setpv(perl_get_sv("@", TRUE), error);
}

static SWIGINLINE void
SWIG_Perl_SetErrorSV(SWIG_MAYBE_PERL_OBJECT SV *error) {
  if (error) sv_setsv(perl_get_sv("@", TRUE), error);
}

static void
SWIG_Perl_SetErrorf(const char *fmt, ...) {
  va_list args;
  va_start(args, fmt);
  sv_vsetpvfn(perl_get_sv("@", TRUE), fmt, strlen(fmt), &args, Null(SV**), 0, Null(bool*));
  va_end(args);
}

/* Macros for low-level exception handling */
#define SWIG_fail       goto fail
#define SWIG_croak(x)   { SWIG_SetError(x); goto fail; }
#define SWIG_croakSV(x) { SWIG_SetErrorSV(x); goto fail; }
/* most preprocessors do not support vararg macros :-( */
/* #define SWIG_croakf(x...) { SWIG_SetErrorf(x); goto fail; } */


typedef XS(SwigPerlWrapper);
typedef SwigPerlWrapper *SwigPerlWrapperPtr;

/* Structure for command table */
typedef struct {
  const char         *name;
  SwigPerlWrapperPtr  wrapper;
} swig_command_info;

/* Information for constant table */

#define SWIG_INT     1
#define SWIG_FLOAT   2
#define SWIG_STRING  3
#define SWIG_POINTER 4
#define SWIG_BINARY  5

/* Constant information structure */
typedef struct swig_constant_info {
    int              type;
    const char      *name;
    long             lvalue;
    double           dvalue;
    void            *pvalue;
    swig_type_info **ptype;
} swig_constant_info;

#ifdef __cplusplus
}
#endif

/* Structure for variable table */
typedef struct {
  const char   *name;
  SwigMagicFunc   set;
  SwigMagicFunc   get;
  swig_type_info  **type;
} swig_variable_info;

/* Magic variable code */
#ifndef PERL_OBJECT
#define swig_create_magic(s,a,b,c) _swig_create_magic(s,a,b,c)
  #ifndef MULTIPLICITY
     static void _swig_create_magic(SV *sv, char *name, int (*set)(SV *, MAGIC *), int (*get)(SV *,MAGIC *)) {
  #else
     static void _swig_create_magic(SV *sv, char *name, int (*set)(struct interpreter*, SV *, MAGIC *), int (*get)(struct interpreter*, SV *,MAGIC *)) {
  #endif
#else
#  define swig_create_magic(s,a,b,c) _swig_create_magic(pPerl,s,a,b,c)
static void _swig_create_magic(CPerlObj *pPerl, SV *sv, const char *name, int (CPerlObj::*set)(SV *, MAGIC *), int (CPerlObj::*get)(SV *, MAGIC *)) {
#endif
  MAGIC *mg;
  sv_magic(sv,sv,'U',(char *) name,strlen(name));
  mg = mg_find(sv,'U');
  mg->mg_virtual = (MGVTBL *) malloc(sizeof(MGVTBL));
  mg->mg_virtual->svt_get = (SwigMagicFuncHack) get;
  mg->mg_virtual->svt_set = (SwigMagicFuncHack) set;
  mg->mg_virtual->svt_len = 0;
  mg->mg_virtual->svt_clear = 0;
  mg->mg_virtual->svt_free = 0;
}






#ifdef do_open
  #undef do_open
#endif
#ifdef do_close
  #undef do_close
#endif
#ifdef scalar
  #undef scalar
#endif
#ifdef list
  #undef list
#endif
#ifdef apply
  #undef apply
#endif
#ifdef convert
  #undef convert
#endif
#ifdef Error
  #undef Error
#endif
#ifdef form
  #undef form
#endif
#ifdef vform
  #undef vform
#endif
#ifdef LABEL
  #undef LABEL
#endif
#ifdef METHOD
  #undef METHOD
#endif
#ifdef Move
  #undef Move
#endif
#ifdef yylex
  #undef yylex
#endif
#ifdef yyparse
  #undef yyparse
#endif
#ifdef yyerror
  #undef yyerror
#endif
#ifdef invert
  #undef invert
#endif
#ifdef ref
  #undef ref
#endif
#ifdef ENTER
  #undef ENTER
#endif


/* -------- TYPES TABLE (BEGIN) -------- */

#define  SWIGTYPE_p_size_t swig_types[0] 
#define  SWIGTYPE_p_iso9660_ltime_t swig_types[1] 
#define  SWIGTYPE_p_iso9660_dtime_t swig_types[2] 
#define  SWIGTYPE_p_unsigned_char swig_types[3] 
#define  SWIGTYPE_p_iso9660_t swig_types[4] 
#define  SWIGTYPE_p_unsigned_int swig_types[5] 
#define  SWIGTYPE_p_cdio_read_mode_t swig_types[6] 
#define  SWIGTYPE_p_char swig_types[7] 
#define  SWIGTYPE_p_CdIo_t swig_types[8] 
#define  SWIGTYPE_p_iso9660_stat_t swig_types[9] 
#define  SWIGTYPE_p_long swig_types[10] 
#define  SWIGTYPE_p_iso9660_pvd_t swig_types[11] 
#define  SWIGTYPE_p_CdioList_t swig_types[12] 
#define  SWIGTYPE_p_iso9660_dir_t swig_types[13] 
#define  SWIGTYPE_p_int swig_types[14] 
static swig_type_info *swig_types[16];

/* -------- TYPES TABLE (END) -------- */

#define SWIG_init    boot_perliso9660

#define SWIG_name   "perliso9660c::boot_perliso9660"
#define SWIG_prefix "perliso9660c::"

#ifdef __cplusplus
extern "C"
#endif
#ifndef PERL_OBJECT
#ifndef MULTIPLICITY
SWIGEXPORT(void) SWIG_init (CV* cv);
#else
SWIGEXPORT(void) SWIG_init (pTHXo_ CV* cv);
#endif
#else
SWIGEXPORT(void) SWIG_init (CV *cv, CPerlObj *);
#endif


/* Includes the header in the wrapper code */
#include <time.h>
#include <cdio/iso9660.h>
#include <cdio/version.h>


/* When libcdio version > 0.76 comes out this won't be needed. */
#include <cdio/version.h>
#if LIBCDIO_VERSION_NUM <= 76

/**< Masks derived from above... */
#undef CDIO_DRIVE_CAP_WRITE_DVD
#define CDIO_DRIVE_CAP_WRITE_DVD (               \
      CDIO_DRIVE_CAP_WRITE_DVD_R                 \
    | CDIO_DRIVE_CAP_WRITE_DVD_PR                \
    | CDIO_DRIVE_CAP_WRITE_DVD_RAM               \
    | CDIO_DRIVE_CAP_WRITE_DVD_RW                \
    | CDIO_DRIVE_CAP_WRITE_DVD_RPW               \
    ) 

/** All the different ways a block/sector can be read. */
typedef enum {
  CDIO_READ_MODE_AUDIO,  /**< CD-DA, audio, Red Book */
  CDIO_READ_MODE_M1F1,   /**< Mode 1 Form 1 */
  CDIO_READ_MODE_M1F2,   /**< Mode 1 Form 2 */
  CDIO_READ_MODE_M2F1,   /**< Mode 2 Form 1 */
  CDIO_READ_MODE_M2F2,   /**< Mode 2 Form 2 */
} cdio_read_mode_t;

/*!
  Reads a number of sectors (AKA blocks).
  
  @param p_buf place to read data into. The caller should make sure
  this location is large enough. See below for size information.
  @param read_mode the kind of "mode" to use in reading.
  @param i_lsn sector to read
  @param i_blocks number of sectors to read
  @return DRIVER_OP_SUCCESS (0) if no error, other (negative) enumerations
  are returned on error.
  
  If read_mode is CDIO_MODE_AUDIO,
    *p_buf should hold at least CDIO_FRAMESIZE_RAW * i_blocks bytes.

  If read_mode is CDIO_MODE_DATA,
    *p_buf should hold at least i_blocks times either ISO_BLOCKSIZE, 
    M1RAW_SECTOR_SIZE or M2F2_SECTOR_SIZE depending on the kind of 
    sector getting read. If you don't know whether you have a Mode 1/2, 
    Form 1/ Form 2/Formless sector best to reserve space for the maximum
    which is M2RAW_SECTOR_SIZE.

  If read_mode is CDIO_MODE_M2F1,
    *p_buf should hold at least M2RAW_SECTOR_SIZE * i_blocks bytes.

  If read_mode is CDIO_MODE_M2F2,
    *p_buf should hold at least CDIO_CD_FRAMESIZE * i_blocks bytes.


*/
driver_return_code_t 
cdio_read_sectors(const CdIo_t *p_cdio, void *p_buf, lsn_t i_lsn, 
                  cdio_read_mode_t read_mode, uint32_t i_blocks)
{
  switch(read_mode) {
  case CDIO_READ_MODE_AUDIO:
    return cdio_read_audio_sectors (p_cdio, p_buf, i_lsn, i_blocks);
  case CDIO_READ_MODE_M1F1:
    return cdio_read_mode1_sectors (p_cdio, p_buf, i_lsn, false, i_blocks);
  case CDIO_READ_MODE_M1F2:
    return cdio_read_mode1_sectors (p_cdio, p_buf, i_lsn, true,  i_blocks);
  case CDIO_READ_MODE_M2F1:
    return cdio_read_mode2_sectors (p_cdio, p_buf, i_lsn, false, i_blocks);
  case CDIO_READ_MODE_M2F2:
    return cdio_read_mode2_sectors (p_cdio, p_buf, i_lsn, true,  i_blocks);
  }
  /* Can't happen. Just to shut up gcc. */
  return DRIVER_OP_ERROR; 
}

driver_return_code_t
cdio_eject_media_drive (const char *psz_drive)
{
  CdIo_t *p_cdio = cdio_open (psz_drive, DRIVER_DEVICE);
  if (p_cdio) {
    return cdio_eject_media(&p_cdio);
  } else {
    return DRIVER_OP_UNINIT;
  }
}
#endif /* LIBCDIO_VERSION_NUM <= 76 */


typedef CdioList_t IsoStatList_t;
typedef iso9660_stat_t IsoStat_t;


  typedef long int my_ssize_t;


typedef char *buf_t;
buf_t seek_read (const iso9660_t *p_iso, 
		 lsn_t start, long int i_size,
		 /*out*/ my_ssize_t *pi_size) {
  
  char *p_buf = calloc(ISO_BLOCKSIZE, i_size);
  *pi_size = iso9660_iso_seek_read(p_iso, p_buf, start, i_size);
  return p_buf;
 }


iso9660_pvd_t *fs_read_pvd ( const CdIo_t *p_cdio ) {
  static iso9660_pvd_t pvd;
  bool b_ok = iso9660_fs_read_pvd ( p_cdio, &pvd );
  if (!b_ok) return NULL;
  return &pvd;
 }


iso9660_pvd_t *ifs_read_pvd ( const iso9660_t *p_iso ) {
  static iso9660_pvd_t pvd;
  bool b_ok = iso9660_ifs_read_pvd ( p_iso, &pvd );
  if (!b_ok) return NULL;
  return &pvd;
 }


iso9660_dtime_t *
set_dtime ( int tm_sec, int tm_min, int tm_hour, int tm_mday, int tm_mon, 
	    int tm_year, int tm_wday, int tm_yday, int tm_isdst)
{
  struct tm tm = { tm_sec, tm_min, tm_hour, tm_mday, tm_mon, tm_year, tm_wday,
		   tm_isdst };
  static iso9660_dtime_t idr_date;
  iso9660_set_dtime (&tm, &idr_date);
  return &idr_date;
}


iso9660_ltime_t *
set_ltime ( int tm_sec, int tm_min, int tm_hour, int tm_mday, int tm_mon, 
	    int tm_year, int tm_wday, int tm_yday, int tm_isdst)

{
  struct tm tm = { tm_sec, tm_min, tm_hour, tm_mday, tm_mon, tm_year, tm_wday,
		   tm_isdst };
  static iso9660_ltime_t pvd_date;
  iso9660_set_ltime (&tm, &pvd_date);
  return &pvd_date;
}


bool get_dtime (const iso9660_dtime_t *idr_date, bool use_localtime,
		int *tm_sec, int *tm_min,  int *tm_hour, int *tm_mday,
		int *tm_mon, int *tm_year, int *tm_wday, int *tm_yday,
		int *tm_isdst) 
{
  struct tm tm;
  bool b_okay = iso9660_get_dtime (idr_date, use_localtime, &tm);
  if (b_okay) {
    *tm_sec   = tm.tm_sec;
    *tm_min   = tm.tm_min;
    *tm_hour  = tm.tm_hour;
    *tm_mon   = tm.tm_mon;
    *tm_mday  = tm.tm_mday;
    *tm_year  = tm.tm_year;
    *tm_wday  = tm.tm_wday;
    *tm_yday  = tm.tm_yday;
    *tm_isdst = tm.tm_isdst;
  }
  return b_okay;
}


bool get_ltime (const iso9660_ltime_t *p_ldate, 
		int *tm_sec, int *tm_min,  int *tm_hour, int *tm_mday,
		int *tm_mon, int *tm_year, int *tm_wday, int *tm_yday,
		int *tm_isdst) 
{
  struct tm tm;
  bool b_okay = iso9660_get_ltime (p_ldate, &tm);
  if (b_okay) {
    *tm_sec   = tm.tm_sec;
    *tm_min   = tm.tm_min;
    *tm_hour  = tm.tm_hour;
    *tm_mon   = tm.tm_mon;
    *tm_mday  = tm.tm_mday;
    *tm_year  = tm.tm_year;
    *tm_wday  = tm.tm_wday;
    *tm_yday  = tm.tm_yday;
    *tm_isdst = tm.tm_isdst;
  }
  return b_okay;
}


char *
name_translate(const char *psz_oldname) {
  char *psz_newname=calloc(sizeof(char), strlen(psz_oldname));
  iso9660_name_translate(psz_oldname, psz_newname);
  return psz_newname;
}


char * 
name_translate_ext(const char *psz_oldname, uint8_t i_joliet_level) {
  char *psz_newname=calloc(sizeof(char), strlen(psz_oldname));
  iso9660_name_translate_ext(psz_oldname, psz_newname, i_joliet_level);
  return psz_newname;
}


char *
strncpy_pad(const char src[], size_t len, enum strncpy_pad_check _check) {
  char *dst = calloc(sizeof(char), len);
  return iso9660_strncpy_pad(dst, src, len, _check);
}


IsoStatList_t *fs_readdir (CdIo_t *p_cdio, const char psz_path[], 
			   bool b_mode2)
{
  CdioList_t *p_statlist = iso9660_fs_readdir (p_cdio, psz_path, b_mode2);
  return p_statlist;
}


IsoStatList_t *ifs_readdir (iso9660_t *p_iso, const char psz_path[])
{
  CdioList_t *p_statlist = iso9660_ifs_readdir (p_iso, psz_path);
  return p_statlist;
}


char *
ifs_get_application_id(iso9660_t *p_iso) {
  char *psz;
  bool ok = iso9660_ifs_get_application_id(p_iso, &psz);
  if (!ok) return NULL;
  return psz;
}


char *
ifs_get_preparer_id(iso9660_t *p_iso) {
  char *psz;
  bool ok = iso9660_ifs_get_preparer_id(p_iso, &psz);
  if (!ok) return NULL;
  return psz;
}


char *
ifs_get_publisher_id(iso9660_t *p_iso) {
  char *psz;
  bool ok = iso9660_ifs_get_publisher_id(p_iso, &psz);
  if (!ok) return NULL;
  return psz;
}


char *
ifs_get_system_id(iso9660_t *p_iso) {
  char *psz;
  bool ok = iso9660_ifs_get_system_id(p_iso, &psz);
  if (!ok) return NULL;
  return psz;
}


char *
ifs_get_volume_id(iso9660_t *p_iso) {
  char *psz;
  bool ok = iso9660_ifs_get_volume_id(p_iso, &psz);
  if (!ok) return NULL;
  return psz;
}


char *
ifs_get_volumeset_id(iso9660_t *p_iso) {
  char *psz;
  bool ok = iso9660_ifs_get_volumeset_id(p_iso, &psz);
  if (!ok) return NULL;
  return psz;
}

#ifdef PERL_OBJECT
#define MAGIC_CLASS _wrap_perliso9660_var::
class _wrap_perliso9660_var : public CPerlObj {
public:
#else
#define MAGIC_CLASS
#endif
SWIGCLASS_STATIC int swig_magic_readonly(pTHX_ SV *sv, MAGIC *mg) {
    MAGIC_PPERL
    sv = sv; mg = mg;
    croak("Value is read-only.");
    return 0;
}


#ifdef PERL_OBJECT
};
#endif

#ifdef __cplusplus
extern "C" {
#endif
XS(_wrap_cdio_read_sectors) {
    {
        CdIo_t *arg1 = (CdIo_t *) 0 ;
        void *arg2 = (void *) 0 ;
        lsn_t arg3 ;
        cdio_read_mode_t arg4 ;
        unsigned int arg5 ;
        driver_return_code_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 5) || (items > 5)) {
            SWIG_croak("Usage: cdio_read_sectors(p_cdio,p_buf,i_lsn,read_mode,i_blocks);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_CdIo_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of cdio_read_sectors. Expected _p_CdIo_t");
            }
        }
        {
            if (SWIG_ConvertPtr(ST(1), (void **) &arg2, 0,0) < 0) {
                SWIG_croak("Type error in argument 2 of cdio_read_sectors. Expected _p_void");
            }
        }
        arg3 = (lsn_t) SvIV(ST(2));
        arg4 = (cdio_read_mode_t) SvIV(ST(3));
        arg5 = (unsigned int) SvUV(ST(4));
        result = (driver_return_code_t)cdio_read_sectors((CdIo_t const *)arg1,arg2,arg3,(cdio_read_mode_t )arg4,arg5);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_cdio_eject_media_drive) {
    {
        char *arg1 = (char *) 0 ;
        driver_return_code_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: cdio_eject_media_drive(psz_drive);");
        }
        if (!SvOK((SV*) ST(0))) arg1 = 0;
        else arg1 = (char *) SvPV(ST(0), PL_na);
        result = (driver_return_code_t)cdio_eject_media_drive((char const *)arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_open_iso) {
    {
        char *arg1 = (char *) 0 ;
        iso9660_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: open_iso(psz_path);");
        }
        if (!SvOK((SV*) ST(0))) arg1 = 0;
        else arg1 = (char *) SvPV(ST(0), PL_na);
        result = (iso9660_t *)iso9660_open((char const *)arg1);
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_iso9660_t, 0|0);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_open_ext) {
    {
        char *arg1 = (char *) 0 ;
        iso_extension_mask_t arg2 ;
        iso9660_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: open_ext(psz_path,iso_extension_mask);");
        }
        if (!SvOK((SV*) ST(0))) arg1 = 0;
        else arg1 = (char *) SvPV(ST(0), PL_na);
        arg2 = (iso_extension_mask_t) SvUV(ST(1));
        result = (iso9660_t *)iso9660_open_ext((char const *)arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_iso9660_t, 0|0);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_open_fuzzy) {
    {
        char *arg1 = (char *) 0 ;
        unsigned int arg2 ;
        iso9660_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: open_fuzzy(psz_path,i_fuzz);");
        }
        if (!SvOK((SV*) ST(0))) arg1 = 0;
        else arg1 = (char *) SvPV(ST(0), PL_na);
        arg2 = (unsigned int) SvUV(ST(1));
        result = (iso9660_t *)iso9660_open_fuzzy((char const *)arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_iso9660_t, 0|0);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_iso9660_open_fuzzy_ext) {
    {
        char *arg1 = (char *) 0 ;
        iso_extension_mask_t arg2 ;
        unsigned int arg3 ;
        iso9660_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 3) || (items > 3)) {
            SWIG_croak("Usage: iso9660_open_fuzzy_ext(psz_path,iso_extension_mask,i_fuzz);");
        }
        if (!SvOK((SV*) ST(0))) arg1 = 0;
        else arg1 = (char *) SvPV(ST(0), PL_na);
        arg2 = (iso_extension_mask_t) SvUV(ST(1));
        arg3 = (unsigned int) SvUV(ST(2));
        result = (iso9660_t *)iso9660_open_fuzzy_ext((char const *)arg1,arg2,arg3);
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_iso9660_t, 0|0);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_fuzzy_read_superblock) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        iso_extension_mask_t arg2 ;
        unsigned int arg3 ;
        bool result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 3) || (items > 3)) {
            SWIG_croak("Usage: ifs_fuzzy_read_superblock(p_iso,iso_extension_mask,i_fuzz);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_fuzzy_read_superblock. Expected _p_iso9660_t");
            }
        }
        arg2 = (iso_extension_mask_t) SvUV(ST(1));
        arg3 = (unsigned int) SvUV(ST(2));
        result = (bool)iso9660_ifs_fuzzy_read_superblock(arg1,arg2,arg3);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_close) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        bool result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: close(p_iso);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of close. Expected _p_iso9660_t");
            }
        }
        result = (bool)iso9660_close(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_seek_read) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        lsn_t arg2 ;
        long arg3 ;
        my_ssize_t *arg4 = (my_ssize_t *) 0 ;
        buf_t result;
        long temp4 ;
        int argvi = 0;
        dXSARGS;
        
        arg4 = &temp4;
        if ((items < 3) || (items > 3)) {
            SWIG_croak("Usage: seek_read(p_iso,start,i_size);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of seek_read. Expected _p_iso9660_t");
            }
        }
        arg2 = (lsn_t) SvIV(ST(1));
        arg3 = (long) SvIV(ST(2));
        result = (buf_t)seek_read((iso9660_t const *)arg1,arg2,arg3,arg4);
        
        {
            /* result is of type buf_t */
            ST(argvi) = sv_newmortal();
            if (result) {
                /* I don't yet know how to pick up arg4 another way.
                       THIS MEANS ARG3 MUST ALWAYS BE THE SIZE.  */
                sv_setpvn((SV*)ST(argvi++), (char *) result, *arg4);
                /* Don't leak memory. return value was malloc'd by libcdio. */
                /* free(result);   */
            } else {
                sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
            }
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg4));
            argvi++;
        }
        free(result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_fs_read_pvd) {
    {
        CdIo_t *arg1 = (CdIo_t *) 0 ;
        iso9660_pvd_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: fs_read_pvd(p_cdio);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_CdIo_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of fs_read_pvd. Expected _p_CdIo_t");
            }
        }
        result = (iso9660_pvd_t *)fs_read_pvd((CdIo_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_iso9660_pvd_t, 0|0);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_read_pvd) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        iso9660_pvd_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: ifs_read_pvd(p_iso);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_read_pvd. Expected _p_iso9660_t");
            }
        }
        result = (iso9660_pvd_t *)ifs_read_pvd((iso9660_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_iso9660_pvd_t, 0|0);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_fs_read_superblock) {
    {
        CdIo_t *arg1 = (CdIo_t *) 0 ;
        iso_extension_mask_t arg2 ;
        bool result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: fs_read_superblock(p_cdio,iso_extension_mask);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_CdIo_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of fs_read_superblock. Expected _p_CdIo_t");
            }
        }
        arg2 = (iso_extension_mask_t) SvUV(ST(1));
        result = (bool)iso9660_fs_read_superblock(arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_read_superblock) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        iso_extension_mask_t arg2 ;
        bool result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: ifs_read_superblock(p_iso,iso_extension_mask);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_read_superblock. Expected _p_iso9660_t");
            }
        }
        arg2 = (iso_extension_mask_t) SvUV(ST(1));
        result = (bool)iso9660_ifs_read_superblock(arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_set_dtime) {
    {
        int arg1 ;
        int arg2 ;
        int arg3 ;
        int arg4 ;
        int arg5 ;
        int arg6 ;
        int arg7 ;
        int arg8 ;
        int arg9 ;
        iso9660_dtime_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 9) || (items > 9)) {
            SWIG_croak("Usage: set_dtime(tm_sec,tm_min,tm_hour,tm_mday,tm_mon,tm_year,tm_wday,tm_yday,tm_isdst);");
        }
        arg1 = (int) SvIV(ST(0));
        arg2 = (int) SvIV(ST(1));
        arg3 = (int) SvIV(ST(2));
        arg4 = (int) SvIV(ST(3));
        arg5 = (int) SvIV(ST(4));
        arg6 = (int) SvIV(ST(5));
        arg7 = (int) SvIV(ST(6));
        arg8 = (int) SvIV(ST(7));
        arg9 = (int) SvIV(ST(8));
        result = (iso9660_dtime_t *)set_dtime(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9);
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_iso9660_dtime_t, 0|0);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_set_ltime) {
    {
        int arg1 ;
        int arg2 ;
        int arg3 ;
        int arg4 ;
        int arg5 ;
        int arg6 ;
        int arg7 ;
        int arg8 ;
        int arg9 ;
        iso9660_ltime_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 9) || (items > 9)) {
            SWIG_croak("Usage: set_ltime(tm_sec,tm_min,tm_hour,tm_mday,tm_mon,tm_year,tm_wday,tm_yday,tm_isdst);");
        }
        arg1 = (int) SvIV(ST(0));
        arg2 = (int) SvIV(ST(1));
        arg3 = (int) SvIV(ST(2));
        arg4 = (int) SvIV(ST(3));
        arg5 = (int) SvIV(ST(4));
        arg6 = (int) SvIV(ST(5));
        arg7 = (int) SvIV(ST(6));
        arg8 = (int) SvIV(ST(7));
        arg9 = (int) SvIV(ST(8));
        result = (iso9660_ltime_t *)set_ltime(arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9);
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_iso9660_ltime_t, 0|0);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_dtime) {
    {
        iso9660_dtime_t *arg1 = (iso9660_dtime_t *) 0 ;
        bool arg2 ;
        int *arg3 = (int *) 0 ;
        int *arg4 = (int *) 0 ;
        int *arg5 = (int *) 0 ;
        int *arg6 = (int *) 0 ;
        int *arg7 = (int *) 0 ;
        int *arg8 = (int *) 0 ;
        int *arg9 = (int *) 0 ;
        int *arg10 = (int *) 0 ;
        int *arg11 = (int *) 0 ;
        bool result;
        int temp3 ;
        int temp4 ;
        int temp5 ;
        int temp6 ;
        int temp7 ;
        int temp8 ;
        int temp9 ;
        int temp10 ;
        int temp11 ;
        int argvi = 0;
        dXSARGS;
        
        arg3 = &temp3;
        arg4 = &temp4;
        arg5 = &temp5;
        arg6 = &temp6;
        arg7 = &temp7;
        arg8 = &temp8;
        arg9 = &temp9;
        arg10 = &temp10;
        arg11 = &temp11;
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: get_dtime(idr_date,use_localtime);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_dtime_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_dtime. Expected _p_iso9660_dtime_t");
            }
        }
        arg2 = (bool) SvIV(ST(1));
        result = (bool)get_dtime((iso9660_dtime_t const *)arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,arg11);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg3));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg4));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg5));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg6));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg7));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg8));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg9));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg10));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg11));
            argvi++;
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_ltime) {
    {
        iso9660_ltime_t *arg1 = (iso9660_ltime_t *) 0 ;
        int *arg2 = (int *) 0 ;
        int *arg3 = (int *) 0 ;
        int *arg4 = (int *) 0 ;
        int *arg5 = (int *) 0 ;
        int *arg6 = (int *) 0 ;
        int *arg7 = (int *) 0 ;
        int *arg8 = (int *) 0 ;
        int *arg9 = (int *) 0 ;
        int *arg10 = (int *) 0 ;
        bool result;
        int temp2 ;
        int temp3 ;
        int temp4 ;
        int temp5 ;
        int temp6 ;
        int temp7 ;
        int temp8 ;
        int temp9 ;
        int temp10 ;
        int argvi = 0;
        dXSARGS;
        
        arg2 = &temp2;
        arg3 = &temp3;
        arg4 = &temp4;
        arg5 = &temp5;
        arg6 = &temp6;
        arg7 = &temp7;
        arg8 = &temp8;
        arg9 = &temp9;
        arg10 = &temp10;
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_ltime(idr_date);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_ltime_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_ltime. Expected _p_iso9660_ltime_t");
            }
        }
        result = (bool)get_ltime((iso9660_ltime_t const *)arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg2));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg3));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg4));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg5));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg6));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg7));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg8));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg9));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg10));
            argvi++;
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_is_dchar) {
    {
        int arg1 ;
        bool result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: is_dchar(c);");
        }
        arg1 = (int) SvIV(ST(0));
        result = (bool)iso9660_isdchar(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_is_achar) {
    {
        int arg1 ;
        bool result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: is_achar(c);");
        }
        arg1 = (int) SvIV(ST(0));
        result = (bool)iso9660_isachar(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_name_translate) {
    {
        char *arg1 = (char *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: name_translate(psz_oldname);");
        }
        if (!SvOK((SV*) ST(0))) arg1 = 0;
        else arg1 = (char *) SvPV(ST(0), PL_na);
        result = (char *)name_translate((char const *)arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        free(result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_name_translate_ext) {
    {
        char *arg1 = (char *) 0 ;
        unsigned char arg2 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: name_translate_ext(psz_oldname,i_joliet_level);");
        }
        if (!SvOK((SV*) ST(0))) arg1 = 0;
        else arg1 = (char *) SvPV(ST(0), PL_na);
        arg2 = (unsigned char) SvUV(ST(1));
        result = (char *)name_translate_ext((char const *)arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        free(result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_strncpy_pad) {
    {
        char *arg1 ;
        size_t arg2 ;
        enum strncpy_pad_check arg3 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 3) || (items > 3)) {
            SWIG_croak("Usage: strncpy_pad(src,len,_check);");
        }
        arg1 = SvPV(ST(0),PL_na);
        
        arg2 = (size_t) SvUV(ST(1));
        arg3 = (enum strncpy_pad_check) SvIV(ST(2));
        result = (char *)strncpy_pad((char const (*))arg1,arg2,(enum strncpy_pad_check )arg3);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        free(result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_dirname_valid_p) {
    {
        char *arg1 ;
        bool result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: dirname_valid_p(psz_path);");
        }
        arg1 = SvPV(ST(0),PL_na);
        
        result = (bool)iso9660_dirname_valid_p((char const (*))arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_pathname_isofy) {
    {
        char *arg1 ;
        unsigned int arg2 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: pathname_isofy(psz_path,i_version);");
        }
        arg1 = SvPV(ST(0),PL_na);
        
        arg2 = (unsigned int) SvUV(ST(1));
        result = (char *)iso9660_pathname_isofy((char const (*))arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        free(result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_pathname_valid_p) {
    {
        char *arg1 ;
        bool result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: pathname_valid_p(psz_path);");
        }
        arg1 = SvPV(ST(0),PL_na);
        
        result = (bool)iso9660_pathname_valid_p((char const (*))arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_fs_find_lsn) {
    {
        CdIo_t *arg1 = (CdIo_t *) 0 ;
        lsn_t arg2 ;
        IsoStat_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: fs_find_lsn(p_cdio,i_lsn);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_CdIo_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of fs_find_lsn. Expected _p_CdIo_t");
            }
        }
        arg2 = (lsn_t) SvIV(ST(1));
        result = (IsoStat_t *)iso9660_find_fs_lsn(arg1,arg2);
        
        {
            // result is of type IsoStatList_t
            iso9660_stat_t *p_statbuf = result;
            
            if (!result) goto out;
            
            PPCODE:
            /* Have perl compute the length of the string using strlen() */
            XPUSHs(sv_2mortal(newSVpv(p_statbuf->filename, 0)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->lsn)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->size)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->secsize)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->type)));
            argvi += 7;
            free (p_statbuf);
            out: ;
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_fs_stat) {
    {
        CdIo_t *arg1 = (CdIo_t *) 0 ;
        char *arg2 ;
        IsoStat_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: fs_stat(p_cdio,psz_path);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_CdIo_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of fs_stat. Expected _p_CdIo_t");
            }
        }
        arg2 = SvPV(ST(1),PL_na);
        
        result = (IsoStat_t *)iso9660_fs_stat(arg1,(char const (*))arg2);
        
        {
            // result is of type IsoStatList_t
            iso9660_stat_t *p_statbuf = result;
            
            if (!result) goto out;
            
            PPCODE:
            /* Have perl compute the length of the string using strlen() */
            XPUSHs(sv_2mortal(newSVpv(p_statbuf->filename, 0)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->lsn)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->size)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->secsize)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->type)));
            argvi += 7;
            free (p_statbuf);
            out: ;
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_fs_stat_translate) {
    {
        CdIo_t *arg1 = (CdIo_t *) 0 ;
        char *arg2 ;
        bool arg3 ;
        IsoStat_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 3) || (items > 3)) {
            SWIG_croak("Usage: fs_stat_translate(p_cdio,psz_path,b_mode2);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_CdIo_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of fs_stat_translate. Expected _p_CdIo_t");
            }
        }
        arg2 = SvPV(ST(1),PL_na);
        
        arg3 = (bool) SvIV(ST(2));
        result = (IsoStat_t *)iso9660_fs_stat_translate(arg1,(char const (*))arg2,arg3);
        
        {
            // result is of type IsoStatList_t
            iso9660_stat_t *p_statbuf = result;
            
            if (!result) goto out;
            
            PPCODE:
            /* Have perl compute the length of the string using strlen() */
            XPUSHs(sv_2mortal(newSVpv(p_statbuf->filename, 0)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->lsn)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->size)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->secsize)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->type)));
            argvi += 7;
            free (p_statbuf);
            out: ;
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_stat) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        char *arg2 ;
        IsoStat_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: ifs_stat(p_iso,psz_path);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_stat. Expected _p_iso9660_t");
            }
        }
        arg2 = SvPV(ST(1),PL_na);
        
        result = (IsoStat_t *)iso9660_ifs_stat(arg1,(char const (*))arg2);
        
        {
            // result is of type IsoStatList_t
            iso9660_stat_t *p_statbuf = result;
            
            if (!result) goto out;
            
            PPCODE:
            /* Have perl compute the length of the string using strlen() */
            XPUSHs(sv_2mortal(newSVpv(p_statbuf->filename, 0)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->lsn)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->size)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->secsize)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->type)));
            argvi += 7;
            free (p_statbuf);
            out: ;
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_stat_translate) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        char *arg2 ;
        IsoStat_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: ifs_stat_translate(p_iso,psz_path);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_stat_translate. Expected _p_iso9660_t");
            }
        }
        arg2 = SvPV(ST(1),PL_na);
        
        result = (IsoStat_t *)iso9660_ifs_stat_translate(arg1,(char const (*))arg2);
        
        {
            // result is of type IsoStatList_t
            iso9660_stat_t *p_statbuf = result;
            
            if (!result) goto out;
            
            PPCODE:
            /* Have perl compute the length of the string using strlen() */
            XPUSHs(sv_2mortal(newSVpv(p_statbuf->filename, 0)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->lsn)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->size)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->secsize)));
            XPUSHs(sv_2mortal(newSViv(p_statbuf->type)));
            argvi += 7;
            free (p_statbuf);
            out: ;
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_fs_readdir) {
    {
        CdIo_t *arg1 = (CdIo_t *) 0 ;
        char *arg2 ;
        bool arg3 ;
        IsoStatList_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 3) || (items > 3)) {
            SWIG_croak("Usage: fs_readdir(p_cdio,psz_path,b_mode2);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_CdIo_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of fs_readdir. Expected _p_CdIo_t");
            }
        }
        arg2 = SvPV(ST(1),PL_na);
        
        arg3 = (bool) SvIV(ST(2));
        result = (IsoStatList_t *)fs_readdir(arg1,(char const (*))arg2,arg3);
        
        {
            // result is of type IsoStatList_t
            CdioList_t *p_entlist   = result;
            CdioListNode_t *p_entnode;
            unsigned int num = 0;
            
            if (!result) goto out;
            
            PPCODE:
            /* Figure out how many items in the array */
            _CDIO_LIST_FOREACH (p_entnode, p_entlist) {
                num +=5;
            }
            
            /* For each element in the array of strings, create a new
                 * mortalscalar, and stuff it into the above array. */
            _CDIO_LIST_FOREACH (p_entnode, p_entlist) {
                iso9660_stat_t *p_statbuf = 
                (iso9660_stat_t *) _cdio_list_node_data (p_entnode);
                /* Have perl compute the length of the string using strlen() */
                XPUSHs(sv_2mortal(newSVpv(p_statbuf->filename, 0)));
                XPUSHs(sv_2mortal(newSViv(p_statbuf->lsn)));
                XPUSHs(sv_2mortal(newSViv(p_statbuf->size)));
                XPUSHs(sv_2mortal(newSViv(p_statbuf->secsize)));
                XPUSHs(sv_2mortal(newSViv(p_statbuf->type)));
            }
            _cdio_list_free (p_entlist, true);
            argvi += num + 2;
            out: ;
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_readdir) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        char *arg2 ;
        IsoStatList_t *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: ifs_readdir(p_iso,psz_path);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_readdir. Expected _p_iso9660_t");
            }
        }
        arg2 = SvPV(ST(1),PL_na);
        
        result = (IsoStatList_t *)ifs_readdir(arg1,(char const (*))arg2);
        
        {
            // result is of type IsoStatList_t
            CdioList_t *p_entlist   = result;
            CdioListNode_t *p_entnode;
            unsigned int num = 0;
            
            if (!result) goto out;
            
            PPCODE:
            /* Figure out how many items in the array */
            _CDIO_LIST_FOREACH (p_entnode, p_entlist) {
                num +=5;
            }
            
            /* For each element in the array of strings, create a new
                 * mortalscalar, and stuff it into the above array. */
            _CDIO_LIST_FOREACH (p_entnode, p_entlist) {
                iso9660_stat_t *p_statbuf = 
                (iso9660_stat_t *) _cdio_list_node_data (p_entnode);
                /* Have perl compute the length of the string using strlen() */
                XPUSHs(sv_2mortal(newSVpv(p_statbuf->filename, 0)));
                XPUSHs(sv_2mortal(newSViv(p_statbuf->lsn)));
                XPUSHs(sv_2mortal(newSViv(p_statbuf->size)));
                XPUSHs(sv_2mortal(newSViv(p_statbuf->secsize)));
                XPUSHs(sv_2mortal(newSViv(p_statbuf->type)));
            }
            _cdio_list_free (p_entlist, true);
            argvi += num + 2;
            out: ;
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_application_id) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_application_id(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_application_id. Expected _p_iso9660_pvd_t");
            }
        }
        result = (char *)iso9660_get_application_id(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_get_application_id) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: ifs_get_application_id(p_iso);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_get_application_id. Expected _p_iso9660_t");
            }
        }
        result = (char *)ifs_get_application_id(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_joliet_level) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        unsigned char result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_joliet_level(p_iso);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_joliet_level. Expected _p_iso9660_t");
            }
        }
        result = (unsigned char)iso9660_ifs_get_joliet_level(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setuv(ST(argvi++), (UV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_dir_len) {
    {
        iso9660_dir_t *arg1 = (iso9660_dir_t *) 0 ;
        unsigned char result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_dir_len(p_idr);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_dir_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_dir_len. Expected _p_iso9660_dir_t");
            }
        }
        result = (unsigned char)iso9660_get_dir_len((iso9660_dir_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setuv(ST(argvi++), (UV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_iso9660_dir_to_name) {
    {
        iso9660_dir_t *arg1 = (iso9660_dir_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: iso9660_dir_to_name(p_iso9660_dir);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_dir_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of iso9660_dir_to_name. Expected _p_iso9660_dir_t");
            }
        }
        result = (char *)iso9660_dir_to_name((iso9660_dir_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_preparer_id) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_preparer_id(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_preparer_id. Expected _p_iso9660_pvd_t");
            }
        }
        result = (char *)iso9660_get_preparer_id((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_get_preparer_id) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: ifs_get_preparer_id(p_iso);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_get_preparer_id. Expected _p_iso9660_t");
            }
        }
        result = (char *)ifs_get_preparer_id(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        free(result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_publisher_id) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_publisher_id(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_publisher_id. Expected _p_iso9660_pvd_t");
            }
        }
        result = (char *)iso9660_get_publisher_id((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_get_publisher_id) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: ifs_get_publisher_id(p_iso);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_get_publisher_id. Expected _p_iso9660_t");
            }
        }
        result = (char *)ifs_get_publisher_id(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        free(result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_pvd_type) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        unsigned char result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_pvd_type(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_pvd_type. Expected _p_iso9660_pvd_t");
            }
        }
        result = (unsigned char)iso9660_get_pvd_type((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setuv(ST(argvi++), (UV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_pvd_id) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_pvd_id(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_pvd_id. Expected _p_iso9660_pvd_t");
            }
        }
        result = (char *)iso9660_get_pvd_id((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_pvd_space_size) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_pvd_space_size(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_pvd_space_size. Expected _p_iso9660_pvd_t");
            }
        }
        result = (int)iso9660_get_pvd_space_size((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_pvd_block_size) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_pvd_block_size(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_pvd_block_size. Expected _p_iso9660_pvd_t");
            }
        }
        result = (int)iso9660_get_pvd_block_size((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_pvd_version) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_pvd_version(pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_pvd_version. Expected _p_iso9660_pvd_t");
            }
        }
        result = (int)iso9660_get_pvd_version((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_system_id) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_system_id(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_system_id. Expected _p_iso9660_pvd_t");
            }
        }
        result = (char *)iso9660_get_system_id((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_get_system_id) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: ifs_get_system_id(p_iso);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_get_system_id. Expected _p_iso9660_t");
            }
        }
        result = (char *)ifs_get_system_id(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        free(result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_root_lsn) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        lsn_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_root_lsn(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_root_lsn. Expected _p_iso9660_pvd_t");
            }
        }
        result = (lsn_t)iso9660_get_root_lsn((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_volume_id) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_volume_id(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_volume_id. Expected _p_iso9660_pvd_t");
            }
        }
        result = (char *)iso9660_get_volume_id((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_get_volume_id) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: ifs_get_volume_id(p_iso);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_get_volume_id. Expected _p_iso9660_t");
            }
        }
        result = (char *)ifs_get_volume_id(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        free(result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_get_volumeset_id) {
    {
        iso9660_pvd_t *arg1 = (iso9660_pvd_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: get_volumeset_id(p_pvd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_pvd_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of get_volumeset_id. Expected _p_iso9660_pvd_t");
            }
        }
        result = (char *)iso9660_get_volumeset_id((iso9660_pvd_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_ifs_get_volumeset_id) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: ifs_get_volumeset_id(p_iso);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of ifs_get_volumeset_id. Expected _p_iso9660_t");
            }
        }
        result = (char *)ifs_get_volumeset_id(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        } else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        free(result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_pathtable_init) {
    {
        void *arg1 = (void *) 0 ;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: pathtable_init(pt);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, 0,0) < 0) {
                SWIG_croak("Type error in argument 1 of pathtable_init. Expected _p_void");
            }
        }
        iso9660_pathtable_init(arg1);
        
        
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_pathtable_get_size) {
    {
        void *arg1 = (void *) 0 ;
        unsigned int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: pathtable_get_size(pt);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, 0,0) < 0) {
                SWIG_croak("Type error in argument 1 of pathtable_get_size. Expected _p_void");
            }
        }
        result = (unsigned int)iso9660_pathtable_get_size((void const *)arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setuv(ST(argvi++), (UV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_pathtable_l_add_entry) {
    {
        void *arg1 = (void *) 0 ;
        char *arg2 ;
        unsigned int arg3 ;
        unsigned int arg4 ;
        unsigned int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 4) || (items > 4)) {
            SWIG_croak("Usage: pathtable_l_add_entry(pt,name,extent,parent);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, 0,0) < 0) {
                SWIG_croak("Type error in argument 1 of pathtable_l_add_entry. Expected _p_void");
            }
        }
        arg2 = SvPV(ST(1),PL_na);
        
        arg3 = (unsigned int) SvUV(ST(2));
        arg4 = (unsigned int) SvUV(ST(3));
        result = (unsigned int)iso9660_pathtable_l_add_entry(arg1,(char const (*))arg2,arg3,arg4);
        
        ST(argvi) = sv_newmortal();
        sv_setuv(ST(argvi++), (UV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_pathtable_m_add_entry) {
    {
        void *arg1 = (void *) 0 ;
        char *arg2 ;
        unsigned int arg3 ;
        unsigned int arg4 ;
        unsigned int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 4) || (items > 4)) {
            SWIG_croak("Usage: pathtable_m_add_entry(pt,name,extent,parent);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, 0,0) < 0) {
                SWIG_croak("Type error in argument 1 of pathtable_m_add_entry. Expected _p_void");
            }
        }
        arg2 = SvPV(ST(1),PL_na);
        
        arg3 = (unsigned int) SvUV(ST(2));
        arg4 = (unsigned int) SvUV(ST(3));
        result = (unsigned int)iso9660_pathtable_m_add_entry(arg1,(char const (*))arg2,arg3,arg4);
        
        ST(argvi) = sv_newmortal();
        sv_setuv(ST(argvi++), (UV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_set_evd) {
    {
        void *arg1 = (void *) 0 ;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: set_evd(pd);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, 0,0) < 0) {
                SWIG_croak("Type error in argument 1 of set_evd. Expected _p_void");
            }
        }
        iso9660_set_evd(arg1);
        
        
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}


XS(_wrap_is_xa) {
    {
        iso9660_t *arg1 = (iso9660_t *) 0 ;
        bool result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: is_xa(p_iso);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_iso9660_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of is_xa. Expected _p_iso9660_t");
            }
        }
        result = (bool)iso9660_ifs_is_xa((iso9660_t const *)arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        ;
    }
    croak(Nullch);
}



/* -------- TYPE CONVERSION AND EQUIVALENCE RULES (BEGIN) -------- */

static swig_type_info _swigt__p_size_t[] = {{"_p_size_t", 0, "size_t *", 0, 0, 0, 0},{"_p_size_t", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_iso9660_ltime_t[] = {{"_p_iso9660_ltime_t", 0, "iso9660_ltime_t *", 0, 0, 0, 0},{"_p_iso9660_ltime_t", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_iso9660_dtime_t[] = {{"_p_iso9660_dtime_t", 0, "iso9660_dtime_t *", 0, 0, 0, 0},{"_p_iso9660_dtime_t", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_unsigned_char[] = {{"_p_unsigned_char", 0, "unsigned char *|iso_extension_mask_t *", 0, 0, 0, 0},{"_p_unsigned_char", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_iso9660_t[] = {{"_p_iso9660_t", 0, "iso9660_t *", 0, 0, 0, 0},{"_p_iso9660_t", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_unsigned_int[] = {{"_p_unsigned_int", 0, "unsigned int *|cdio_drive_misc_cap_t *", 0, 0, 0, 0},{"_p_unsigned_int", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_cdio_read_mode_t[] = {{"_p_cdio_read_mode_t", 0, "enum cdio_read_mode_t *|cdio_read_mode_t *", 0, 0, 0, 0},{"_p_cdio_read_mode_t", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_char[] = {{"_p_char", 0, "char *", 0, 0, 0, 0},{"_p_char", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_CdIo_t[] = {{"_p_CdIo_t", 0, "CdIo_t *", 0, 0, 0, 0},{"_p_CdIo_t", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_iso9660_stat_t[] = {{"_p_iso9660_stat_t", 0, "iso9660_stat_t *|IsoStat_t *", 0, 0, 0, 0},{"_p_iso9660_stat_t", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_long[] = {{"_p_long", 0, "long *|my_ssize_t *", 0, 0, 0, 0},{"_p_long", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_iso9660_pvd_t[] = {{"_p_iso9660_pvd_t", 0, "iso9660_pvd_t *", 0, 0, 0, 0},{"_p_iso9660_pvd_t", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_CdioList_t[] = {{"_p_CdioList_t", 0, "CdioList_t *|IsoStatList_t *", 0, 0, 0, 0},{"_p_CdioList_t", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_iso9660_dir_t[] = {{"_p_iso9660_dir_t", 0, "iso9660_dir_t *", 0, 0, 0, 0},{"_p_iso9660_dir_t", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};
static swig_type_info _swigt__p_int[] = {{"_p_int", 0, "int *", 0, 0, 0, 0},{"_p_int", 0, 0, 0, 0, 0, 0},{0, 0, 0, 0, 0, 0, 0}};

static swig_type_info *swig_types_initial[] = {
_swigt__p_size_t, 
_swigt__p_iso9660_ltime_t, 
_swigt__p_iso9660_dtime_t, 
_swigt__p_unsigned_char, 
_swigt__p_iso9660_t, 
_swigt__p_unsigned_int, 
_swigt__p_cdio_read_mode_t, 
_swigt__p_char, 
_swigt__p_CdIo_t, 
_swigt__p_iso9660_stat_t, 
_swigt__p_long, 
_swigt__p_iso9660_pvd_t, 
_swigt__p_CdioList_t, 
_swigt__p_iso9660_dir_t, 
_swigt__p_int, 
0
};


/* -------- TYPE CONVERSION AND EQUIVALENCE RULES (END) -------- */

static swig_constant_info swig_constants[] = {
{ SWIG_INT,     (char *) SWIG_prefix "VERSION_NUM", (long) LIBCDIO_VERSION_NUM, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "INVALID_LBA", (long) CDIO_INVALID_LBA, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "INVALID_LSN", (long) CDIO_INVALID_LSN, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "CD_FRAMESIZE", (long) CDIO_CD_FRAMESIZE, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "CD_FRAMESIZE_RAW", (long) CDIO_CD_FRAMESIZE_RAW, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "ISO_BLOCKSIZE", (long) CDIO_CD_FRAMESIZE, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "M2F2_SECTOR_SIZE", (long) M2F2_SECTOR_SIZE, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "M2RAW_SECTOR_SIZE", (long) M2RAW_SECTOR_SIZE, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "CDIO_READ_MODE_AUDIO", (long) CDIO_READ_MODE_AUDIO, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "CDIO_READ_MODE_M1F1", (long) CDIO_READ_MODE_M1F1, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "CDIO_READ_MODE_M1F2", (long) CDIO_READ_MODE_M1F2, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "CDIO_READ_MODE_M2F1", (long) CDIO_READ_MODE_M2F1, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "CDIO_READ_MODE_M2F2", (long) CDIO_READ_MODE_M2F2, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "PVD_SECTOR", (long) ISO_PVD_SECTOR, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "EVD_SECTOR", (long) ISO_EVD_SECTOR, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "LEN_ISONAME", (long) LEN_ISONAME, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "MAX_SYSTEM_ID", (long) ISO_MAX_SYSTEM_ID, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "MAX_ISONAME", (long) MAX_ISONAME, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "MAX_PREPARER_ID", (long) ISO_MAX_PREPARER_ID, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "MAX_ISOPATHNAME", (long) MAX_ISOPATHNAME, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "FILE", (long) ISO_FILE, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "EXISTENCE", (long) ISO_EXISTENCE, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DIRECTORY", (long) ISO_DIRECTORY, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "ASSOCIATED", (long) ISO_ASSOCIATED, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "RECORD", (long) ISO_RECORD, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "PROECTION", (long) ISO_PROECTION, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DRESERVED1", (long) ISO_DRESERVED1, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DRESERVED2", (long) ISO_DRESERVED2, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "MULTIEXTENT", (long) ISO_MULTIEXTENT, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "VD_BOOT_RECORD", (long) ISO_VD_BOOT_RECORD, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "VD_PRIMARY", (long) ISO_VD_PRIMARY, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "VD_SUPPLEMENTARY", (long) ISO_VD_SUPPLEMENTARY, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "VD_PARITION", (long) ISO_VD_PARITION, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "VD_END", (long) ISO_VD_END, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "MAX_PUBLISHER_ID", (long) ISO_MAX_PUBLISHER_ID, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "MAX_APPLICATION_ID", (long) ISO_MAX_APPLICATION_ID, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "MAX_VOLUME_ID", (long) ISO_MAX_VOLUME_ID, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "MAX_VOLUMESET_ID", (long) ISO_MAX_VOLUMESET_ID, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "STANDARD_ID", (long) ISO_STANDARD_ID, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "NOCHECK", (long) ISO9660_NOCHECK, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "SEVEN_BIT", (long) ISO9660_7BIT, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "ACHARS", (long) ISO9660_ACHARS, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DCHARS", (long) ISO9660_DCHARS, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "EXTENSION_JOLIET_LEVEL1", (long) ISO_EXTENSION_JOLIET_LEVEL1, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "EXTENSION_JOLIET_LEVEL2", (long) ISO_EXTENSION_JOLIET_LEVEL2, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "EXTENSION_JOLIET_LEVEL3", (long) ISO_EXTENSION_JOLIET_LEVEL3, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "EXTENSION_ROCK_RIDGE", (long) ISO_EXTENSION_ROCK_RIDGE, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "EXTENSION_HIGH_SIERRA", (long) ISO_EXTENSION_HIGH_SIERRA, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "EXTENSION_ALL", (long) ISO_EXTENSION_ALL, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "EXTENSION_NONE", (long) ISO_EXTENSION_NONE, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "EXTENSION_JOLIET", (long) ISO_EXTENSION_JOLIET, 0, 0, 0},
{0,0,0,0,0,0}
};
#ifdef __cplusplus
}
#endif
static swig_variable_info swig_variables[] = {
{0,0,0,0}
};
static swig_command_info swig_commands[] = {
{"perliso9660c::cdio_read_sectors", _wrap_cdio_read_sectors},
{"perliso9660c::cdio_eject_media_drive", _wrap_cdio_eject_media_drive},
{"perliso9660c::open_iso", _wrap_open_iso},
{"perliso9660c::open_ext", _wrap_open_ext},
{"perliso9660c::open_fuzzy", _wrap_open_fuzzy},
{"perliso9660c::iso9660_open_fuzzy_ext", _wrap_iso9660_open_fuzzy_ext},
{"perliso9660c::ifs_fuzzy_read_superblock", _wrap_ifs_fuzzy_read_superblock},
{"perliso9660c::close", _wrap_close},
{"perliso9660c::seek_read", _wrap_seek_read},
{"perliso9660c::fs_read_pvd", _wrap_fs_read_pvd},
{"perliso9660c::ifs_read_pvd", _wrap_ifs_read_pvd},
{"perliso9660c::fs_read_superblock", _wrap_fs_read_superblock},
{"perliso9660c::ifs_read_superblock", _wrap_ifs_read_superblock},
{"perliso9660c::set_dtime", _wrap_set_dtime},
{"perliso9660c::set_ltime", _wrap_set_ltime},
{"perliso9660c::get_dtime", _wrap_get_dtime},
{"perliso9660c::get_ltime", _wrap_get_ltime},
{"perliso9660c::is_dchar", _wrap_is_dchar},
{"perliso9660c::is_achar", _wrap_is_achar},
{"perliso9660c::name_translate", _wrap_name_translate},
{"perliso9660c::name_translate_ext", _wrap_name_translate_ext},
{"perliso9660c::strncpy_pad", _wrap_strncpy_pad},
{"perliso9660c::dirname_valid_p", _wrap_dirname_valid_p},
{"perliso9660c::pathname_isofy", _wrap_pathname_isofy},
{"perliso9660c::pathname_valid_p", _wrap_pathname_valid_p},
{"perliso9660c::fs_find_lsn", _wrap_fs_find_lsn},
{"perliso9660c::fs_stat", _wrap_fs_stat},
{"perliso9660c::fs_stat_translate", _wrap_fs_stat_translate},
{"perliso9660c::ifs_stat", _wrap_ifs_stat},
{"perliso9660c::ifs_stat_translate", _wrap_ifs_stat_translate},
{"perliso9660c::fs_readdir", _wrap_fs_readdir},
{"perliso9660c::ifs_readdir", _wrap_ifs_readdir},
{"perliso9660c::get_application_id", _wrap_get_application_id},
{"perliso9660c::ifs_get_application_id", _wrap_ifs_get_application_id},
{"perliso9660c::get_joliet_level", _wrap_get_joliet_level},
{"perliso9660c::get_dir_len", _wrap_get_dir_len},
{"perliso9660c::iso9660_dir_to_name", _wrap_iso9660_dir_to_name},
{"perliso9660c::get_preparer_id", _wrap_get_preparer_id},
{"perliso9660c::ifs_get_preparer_id", _wrap_ifs_get_preparer_id},
{"perliso9660c::get_publisher_id", _wrap_get_publisher_id},
{"perliso9660c::ifs_get_publisher_id", _wrap_ifs_get_publisher_id},
{"perliso9660c::get_pvd_type", _wrap_get_pvd_type},
{"perliso9660c::get_pvd_id", _wrap_get_pvd_id},
{"perliso9660c::get_pvd_space_size", _wrap_get_pvd_space_size},
{"perliso9660c::get_pvd_block_size", _wrap_get_pvd_block_size},
{"perliso9660c::get_pvd_version", _wrap_get_pvd_version},
{"perliso9660c::get_system_id", _wrap_get_system_id},
{"perliso9660c::ifs_get_system_id", _wrap_ifs_get_system_id},
{"perliso9660c::get_root_lsn", _wrap_get_root_lsn},
{"perliso9660c::get_volume_id", _wrap_get_volume_id},
{"perliso9660c::ifs_get_volume_id", _wrap_ifs_get_volume_id},
{"perliso9660c::get_volumeset_id", _wrap_get_volumeset_id},
{"perliso9660c::ifs_get_volumeset_id", _wrap_ifs_get_volumeset_id},
{"perliso9660c::pathtable_init", _wrap_pathtable_init},
{"perliso9660c::pathtable_get_size", _wrap_pathtable_get_size},
{"perliso9660c::pathtable_l_add_entry", _wrap_pathtable_l_add_entry},
{"perliso9660c::pathtable_m_add_entry", _wrap_pathtable_m_add_entry},
{"perliso9660c::set_evd", _wrap_set_evd},
{"perliso9660c::is_xa", _wrap_is_xa},
{0,0}
};


static void SWIG_Perl_SetTypeListHandle(swig_type_info **handle) {
    SV *pointer;
    
    /* create a new pointer */
    pointer = get_sv("swig_runtime_data::type_pointer" SWIG_RUNTIME_VERSION SWIG_TYPE_TABLE_NAME, TRUE);
    sv_setiv(pointer, PTR2IV(swig_type_list_handle));
}

static swig_type_info **
SWIG_Perl_LookupTypePointer(swig_type_info **type_list_handle) {
    swig_type_info **type_pointer;
    
    /* first check if module already created */
    type_pointer = SWIG_Perl_GetTypeListHandle();
    if (type_pointer) {
        return type_pointer;
    } else {
        /* create a new module and variable */
        SWIG_Perl_SetTypeListHandle(type_list_handle);
        return type_list_handle;
    }
}


#ifdef __cplusplus
extern "C"
#endif

XS(SWIG_init) {
    dXSARGS;
    int i;
    static int _init = 0;
    if (!_init) {
        swig_type_list_handle = SWIG_Perl_LookupTypePointer(swig_type_list_handle);
        for (i = 0; swig_types_initial[i]; i++) {
            swig_types[i] = SWIG_TypeRegister(swig_types_initial[i]);
        }	
        _init = 1;
    }
    
    /* Install commands */
    for (i = 0; swig_commands[i].name; i++) {
        newXS((char*) swig_commands[i].name,swig_commands[i].wrapper, (char*)__FILE__);
    }
    
    /* Install variables */
    for (i = 0; swig_variables[i].name; i++) {
        SV *sv;
        sv = perl_get_sv((char*) swig_variables[i].name, TRUE | 0x2);
        if (swig_variables[i].type) {
            SWIG_MakePtr(sv,(void *)1, *swig_variables[i].type,0);
        } else {
            sv_setiv(sv,(IV) 0);
        }
        swig_create_magic(sv, (char *) swig_variables[i].name, swig_variables[i].set, swig_variables[i].get); 
    }
    
    /* Install constant */
    for (i = 0; swig_constants[i].type; i++) {
        SV *sv;
        sv = perl_get_sv((char*)swig_constants[i].name, TRUE | 0x2);
        switch(swig_constants[i].type) {
            case SWIG_INT:
            sv_setiv(sv, (IV) swig_constants[i].lvalue);
            break;
            case SWIG_FLOAT:
            sv_setnv(sv, (double) swig_constants[i].dvalue);
            break;
            case SWIG_STRING:
            sv_setpv(sv, (char *) swig_constants[i].pvalue);
            break;
            case SWIG_POINTER:
            SWIG_MakePtr(sv, swig_constants[i].pvalue, *(swig_constants[i].ptype),0);
            break;
            case SWIG_BINARY:
            SWIG_MakePackedObj(sv, swig_constants[i].pvalue, swig_constants[i].lvalue, *(swig_constants[i].ptype));
            break;
            default:
            break;
        }
        SvREADONLY_on(sv);
    }
    
    ST(0) = &PL_sv_yes;
    XSRETURN(1);
}

